#pragma once

#include "Arduino.h"
#include "VFileSystem.h"

#define RECORD_COUNT 65536
#define RECORD_PER_BLOCK 64
#define BLOCK_COUNT (RECORD_COUNT / RECORD_PER_BLOCK)
#define RECORD_TOTAL_SIZE 96
#define RECORD_DATA_SIZE 64
#define RECORD_LEN_SIZE 1
#define RECORD_CRC_SIZE 2
#define RECORD_DUMMY_SIZE (RECORD_TOTAL_SIZE - RECORD_DATA_SIZE - RECORD_LEN_SIZE - RECORD_CRC_SIZE)
#define RECORD_BLOCK_SIZE (RECORD_TOTAL_SIZE * RECORD_PER_BLOCK)

#define FORCESAVE_READ_COUNT 64
#define FORCESAVE_WRITE_COUNT 8

class DataRecorder
{
public:
    int Init(VFileSystem* fs);
    int PushData(uint8_t len, const uint8_t* data);
    int PopData(int& index, uint8_t& len, uint8_t* data);
    int ReadData(int& index, uint8_t& len, uint8_t* data);
    int ReadDataFromIndex(int index, uint8_t& len, uint8_t* data);
    int HeadIndex();
    int TailIndex();
    int RecordCount();
    int ForceSave();

private:
    int ReadPosition();
    int WritePosition();
    int IncHeadIndex();
    int IncTailIndex();
    bool HeadTailSameBlock();
    void SyncHeadTailBlock();
    int ReadHeadRecordBlock(int blockIndex);
    int ReadTailRecordBlock(int blockIndex);
    int ReadTempRecordBlock(int blockIndex);
    int WriteTailRecordBlock(int blockIndex);

private:
    VFileSystem* fs;
    int head;
    int tail;
    int tempReadBlockIndex;
};
