#include "InterSerial.h"
#include "ConfigManager.h"

#define MAX_CMD_LEN 200

void InterSerial::Init(DataRecorder* r, ConfigManager* c)
{
    Serial1.begin(115200, SERIAL_8N1, 18, 17);
    cmdLine = "";
    rec = r;
    cfg = c;
}

void InterSerial::Process()
{
    while (Serial1.available()) {
        ringBuffer.push(Serial1.read());
    }

    while (ringBuffer.size() > 0) {
        if (cmdLine.endsWith("\r\n")) {
            cmdLine.clear();
        }
        
        uint8_t readByte;
        ringBuffer.pop(readByte);
        if (cmdLine.isEmpty() && isspace(readByte)) {
            continue;
        }
        cmdLine += (char)readByte;

        if (cmdLine.length() > MAX_CMD_LEN) {
            Serial1.printf("Command too long, max 200 bytes\r\n");
            cmdLine.clear();
            continue;
        }
        if (cmdLine.endsWith("\r\n")) {
            if (cmdLine.startsWith("AT")) {
                CmdHandler(cmdLine);
            } else {
                Serial1.printf("Command should start with AT\r\n");
                cmdLine.clear();
                continue;
            }
        }
    }
}

void InterSerial::CmdHandler(String cmdLine)
{
    if (cmdLine == "AT\r\n") {
        Serial1.printf("OK\r\n");
    } else if (cmdLine.length() <= 5) {
        Serial1.printf("Bad Command\r\n");
    } else if (cmdLine[strlen("AT")] != '+') {
        Serial1.printf("Bad Command\r\n");
    } else {
        String realCmd = cmdLine.substring(3, cmdLine.length() - 2);
        ParseCmd(realCmd);
    }
}

void InterSerial::ParseCmd(String cmd)
{
    if (cmd == "DEVIDREAD=?") {
        String id;
        if (cfg->ReadId(id)) {
            Serial1.printf("ERROR\r\n");
            return;
        }
        Serial1.printf("%s\n", id.c_str());

    } else if (cmd == "SYSCFGREAD=?") {
        String sysconfig;
        if (cfg->ReadConfig(sysconfig)) {
            Serial1.printf("ERROR\r\n");
            return;
        }
        Serial1.printf("%s\n", sysconfig.c_str());

    } else if (cmd.startsWith("MSGSTORE") && cmd[strlen("MSGSTORE")] == '=') {
        String value = cmd.substring(strlen("MSGSTORE") + 1);
        int i;
        for (i = 0; i < value.length(); i++) {
            if (value[i] == ',') {
                break;
            }
        }
        if (i >= value.length() - 1) {
            Serial1.printf("ERROR\r\n");
            return;
        }
        String dataLenStr = value.substring(0, i);
        String dataBufStr = value.substring(i + 1);
        int dataLen = (int)dataLenStr.toInt();
        if (dataLen == 0 || dataLen > RECORD_DATA_SIZE) {
            Serial1.printf("ERROR\r\n");
            return;
        }
        if (dataLen * 2 != dataBufStr.length()) {
            Serial1.printf("ERROR\r\n");
            return;
        }
        uint8_t dataBuf[dataLen] = {0};
        for (i = 0; i < dataLen; i++) {
            String tmp = dataBufStr.substring(i * 2, i * 2 + 2);
            dataBuf[i] = strtol(tmp.c_str(), 0, 16);
        }
        if (rec->PushData(dataLen, dataBuf)) {
            Serial1.printf("ERROR\r\n");
            return;
        }
        Serial1.printf("OK\r\n");

    } else if (cmd == "MSGREAD=?") {
        uint8_t dataBuf[RECORD_DATA_SIZE] = {0};
        uint8_t dataLen = 0;
        int index = 0;
        if (rec->ReadData(index, dataLen, dataBuf)) {
            Serial1.printf("ERROR\r\n");
            return;
        }
        String result = "";
        result += String(index);
        result += ",";
        result += String(dataLen);
        result += ",";
        char tmpHex[2] = {0};
        for (int i = 0; i < dataLen; i++) {
            sprintf(tmpHex, "%02X", dataBuf[i]);
            result += tmpHex[0];
            result += tmpHex[1];
        }
        Serial1.printf("%s\r\n", result.c_str());

    } else if (cmd == "MSGPOP=?") {
        uint8_t dataBuf[RECORD_DATA_SIZE] = {0};
        uint8_t dataLen = 0;
        int index = 0;
        if (rec->PopData(index, dataLen, dataBuf)) {
            Serial1.printf("ERROR\r\n");
            return;
        }
        String result = "";
        result += String(index);
        result += ",";
        result += String(dataLen);
        result += ",";
        char tmpHex[2] = {0};
        for (int i = 0; i < dataLen; i++) {
            sprintf(tmpHex, "%02X", dataBuf[i]);
            result += tmpHex[0];
            result += tmpHex[1];
        }
        Serial1.printf("%s\r\n", result.c_str());

    } else if (cmd == "FORCESAVE") {
        if (rec->ForceSave()) {
            Serial1.printf("ERROR\r\n");
        } else {
            Serial1.printf("OK\r\n");
        }

    } else {
        Serial1.printf("Bad Command\r\n");
    }
}
