#include "UpperSerial.h"

#define MAX_CMD_LEN 200

void UpperSerial::Init(DataRecorder* r, ConfigManager* c)
{
    Serial.begin();
    cmdLine = "";
    rec = r;
    cfg = c;
}

void UpperSerial::Process()
{
    while (Serial.available()) {
        ringBuffer.push(Serial.read());
    }

    while (ringBuffer.size() > 0) {
        if (cmdLine.endsWith("\r\n")) {
            cmdLine.clear();
        }
        
        uint8_t readByte;
        ringBuffer.pop(readByte);
        if (cmdLine.isEmpty() && isspace(readByte)) {
            continue;
        }
        cmdLine += (char)readByte;

        if (cmdLine.length() > MAX_CMD_LEN) {
            Serial.printf("Command too long, max 200 bytes\r\n");
            cmdLine.clear();
            continue;
        }
        if (cmdLine.endsWith("\r\n")) {
            if (cmdLine.startsWith("AT")) {
                CmdHandler(cmdLine);
            } else {
                Serial.printf("Command should start with AT\r\n");
                cmdLine.clear();
                continue;
            }
        }
    }
}

void UpperSerial::CmdHandler(String cmdLine)
{
    if (cmdLine == "AT\r\n") {
        Serial.printf("OK\r\n");
    } else if (cmdLine.length() <= 5) {
        Serial.printf("Bad Command\r\n");
    } else if (cmdLine[strlen("AT")] != '+') {
        Serial.printf("Bad Command\r\n");
    } else {
        String realCmd = cmdLine.substring(3, cmdLine.length() - 2);
        ParseCmd(realCmd);
    }
}

void UpperSerial::ParseCmd(String cmd)
{
    if (cmd == "DEVIDREAD=?") {
        String id;
        if (cfg->ReadId(id)) {
            Serial.printf("ERROR\r\n");
            return;
        }
        Serial.printf("%s\n", id.c_str());

    } else if (cmd == "SYSCFGREAD=?") {
        String sysconfig;
        if (cfg->ReadConfig(sysconfig)) {
            Serial.printf("ERROR\r\n");
            return;
        }
        Serial.printf("%s\n", sysconfig.c_str());

    } else if (cmd.startsWith("DEVIDSET") && cmd[strlen("DEVIDSET")] == '=') {
        String value = cmd.substring(strlen("DEVIDSET") + 1);
        if (cfg->SetId(value)) {
            Serial.printf("ERROR\r\n");
        } else {
            Serial.printf("OK\r\n");
        }

    } else if (cmd.startsWith("SYSCFGSET") && cmd[strlen("SYSCFGSET")] == '=') {
        String value = cmd.substring(strlen("SYSCFGSET") + 1);
        if (cfg->WriteConfig(value)) {
            Serial.printf("ERROR\r\n");
        } else {
            Serial.printf("OK\r\n");
        }

    } else if (cmd == "MSGREAD=?") {
        for (int i = 0; i < 100; i++) {
            uint8_t dataBuf[RECORD_DATA_SIZE] = {0};
            uint8_t dataLen = 0;
            int index = 0;
            int ret = rec->PopData(index, dataLen, dataBuf);
            if (ret != 0) {
                if (ret == -4) {
                    Serial.printf("END\r\n");
                } else {
                    Serial.printf("ERROR\r\n");
                }
                return;
            }
            String result = "";
            result += String(index);
            result += ",";
            result += String(dataLen);
            result += ",";
            char tmpHex[2] = {0};
            for (int i = 0; i < dataLen; i++) {
                sprintf(tmpHex, "%02X", dataBuf[i]);
                result += tmpHex[0];
                result += tmpHex[1];
            }
            Serial.printf("%s\r\n", result.c_str());
        }

    } else if (cmd.startsWith("MSGREAD") && cmd[strlen("MSGREAD")] == '=') {
        String value = cmd.substring(strlen("MSGREAD") + 1);
        int startIndex = (int)value.toInt();
        if (startIndex >= RECORD_COUNT - 100 || startIndex < 0) {
            Serial.printf("ERROR\r\n");
            return;
        }
        for (int i = startIndex; i < startIndex + 100; i++) {
            uint8_t dataBuf[RECORD_DATA_SIZE] = {0};
            uint8_t dataLen = 0;
            int ret;
            if (ret = rec->ReadDataFromIndex(i, dataLen, dataBuf)) {
                Serial.printf("ERROR\r\n");
                return;
            }
            String result = "";
            result += String(i);
            result += ",";
            result += String(dataLen);
            result += ",";
            char tmpHex[2] = {0};
            for (int i = 0; i < dataLen; i++) {
                sprintf(tmpHex, "%02X", dataBuf[i]);
                result += tmpHex[0];
                result += tmpHex[1];
            }
            Serial.printf("%s\r\n", result.c_str());
        }

    } else if (cmd == "RECHEADINDEX=?") {
        Serial.printf("%d\r\n", rec->HeadIndex());

    } else if (cmd == "RECTAILINDEX=?") {
        Serial.printf("%d\r\n", rec->TailIndex());

    } else if (cmd == "RECCOUNT=?") {
        Serial.printf("%d\r\n", rec->RecordCount());

    } else if (cmd == "FORCESAVE") {
        if (rec->ForceSave()) {
            Serial.printf("ERROR\r\n");
        } else {
            Serial.printf("OK\r\n");
        }

    } else {
        Serial.printf("Bad Command\r\n");
    }
}
