#include "VFileSystem.h"
#include <FS.h>
#include <SPIFFS.h>

int VFileSystem::Init()
{
    if (!SPIFFS.begin(FORMAT_SPIFFS_IF_FAILED)) {
        return -1;
    }

    return 0;
}

size_t VFileSystem::TotalSpace()
{
    return SPIFFS.totalBytes();
}

size_t VFileSystem::FreeSpace()
{
    return SPIFFS.totalBytes() - SPIFFS.usedBytes();
}

int VFileSystem::ReadFile(const char* fileName, uint8_t* result, int len)
{
    File file = SPIFFS.open(fileName, FILE_READ);
    if (!file || file.isDirectory()) {
        return -1;
    }
    
    int filePos = 0;
    int ret = 0;
    while (file.available()) {
        if (filePos == len) {
            ret = -2;
            break;
        }
        result[filePos] = file.read();
        filePos++;
    }
    file.close();

    return ret;
}

int VFileSystem::WriteFile(const char* fileName, const uint8_t* data, int len)
{
    File file = SPIFFS.open(fileName, FILE_WRITE);
    if (!file) {
        return -1;
    }

    int ret = 0;
    for (int i = 0; i < len; i++) {
        if (file.write(data[i]) != 1) {
            ret = -1;
            break;
        }
    }
    file.close();

    return ret;
}
